## Game section

## Contents

- [Snake](#snake)
- [Conway's Game of Life](#conway_s_game_of_life)
- [Conway's Game of Life](#conways_game_of_life)
- [Khammurapi](#khammurapi)
  
### Snake

https://codeberg.org/DW0RKiN/M4_FORTH/src/branch/master/Game/snake.m4
![Snake Game screenshot](https://codeberg.org/DW0RKiN/M4_FORTH/raw/branch/master/Game/snake.png)

Eat the food at the coordinate point, but don't eat yourself!

    q   up
    a   down
    o   left
    p   right

### Conway's Game of Life

https://codeberg.org/DW0RKiN/M4_FORTH/src/branch/master/Game/life.m4
![Conway's Game of Life screenshot](https://codeberg.org/DW0RKiN/M4_FORTH/raw/branch/master/Game/life.png)

    q   up
    a   down
    o   left
    p   right
    s   swap cell
    c   clear all
    f   fill all
    e   exit

Rules:
- Any live cell with two or three live neighbours survives. 
- Any dead cell with three live neighbours becomes a live cell.
- All other live cells die in the next generation. Similarly, all other dead cells stay dead.

Endless playing area. One edge follows the other edge.
So I had to take care of this when underlining the left edge:

    dup 1- 0x1F and swap 0xFFE0 and +
or if applicable at the overflow of the right edge:

    dup 1+ 0x1F and swap 0xFFE0 and +
Where the verbal description of this piece of code is: make a copy of a 16-bit address, add or subtract one, leave only the lower 5 bits, swap with the original, reset the lower 5 bits and add the numbers.

And because the whole program is an M4 macro, I could immediately create the words LEFT and RIGHT in the program and use them. It looks clear and it's much more powerful and shorter, because I only work with an 8-bit value.

    ; dup 1- 0x1F and swap 0xFFE0 and +
    define({LEFT},{
        ld    A, L          ; 1:4       left
        dec   A             ; 1:4       left
        xor   L             ; 1:4       left
        and  0x1F           ; 2:7       left
        xor   L             ; 1:4       left
        ld    L, A          ; 1:4       left})dnl

    ; dup 1+ 0x1F and swap 0xFFE0 and +
    define({RIGHT},{
        ld    A, L          ; 1:4       right
        inc   A             ; 1:4       right
        xor   L             ; 1:4       right
        and  0x1F           ; 2:7       right
        xor   L             ; 1:4       right
        ld    L, A          ; 1:4       right})dnl
        
Correction: I have now tokenized the compiler, so creating the macro itself is not enough, because it will not be tokenized and will only generate instructions before the program itself. 
So the word __TOKEN_ + NAME token must be created first and the original macro is renamed to __ASM_TOKEN_ + NAME, it will be called by the token at the very end, once the rules for tokens are finished and the code itself starts to be generated.

    ;# dup 1- 0x1F and swap 0xFFE0 and + 
    define({LEFT},{dnl
    __{}__ADD_TOKEN({__TOKEN_LEFT},{<--},$@){}dnl
    }){}dnl
    dnl
    define({__ASM_TOKEN_LEFT},{dnl
    __{}define({__INFO},__COMPILE_INFO)
        ld    A, L          ; 1:4       __INFO
        dec   A             ; 1:4       __INFO
        xor   L             ; 1:4       __INFO
        and  0x1F           ; 2:7       __INFO
        xor   L             ; 1:4       __INFO
        ld    L, A          ; 1:4       __INFO})dnl
    
    ;# dup 1+ 0x1F and swap 0xFFE0 and + 
    define({RIGHT},{dnl
    __{}__ADD_TOKEN({__TOKEN_RIGHT},{-->},$@){}dnl
    }){}dnl
    dnl
    define({__ASM_TOKEN_RIGHT},{dnl
    __{}define({__INFO},__COMPILE_INFO)
        ld    A, L          ; 1:4       __INFO
        inc   A             ; 1:4       __INFO
        xor   L             ; 1:4       __INFO
        and  0x1F           ; 2:7       __INFO
        xor   L             ; 1:4       __INFO
        ld    L, A          ; 1:4       __INFO})dnl
    
    define({SWAP_CURSOR},{dnl
    __{}__ADD_TOKEN({__TOKEN_SWAP_CURSOR},{swap_cursor},$@){}dnl
    }){}dnl
    dnl
    define({__ASM_TOKEN_SWAP_CURSOR},{dnl
    __{}define({__INFO},__COMPILE_INFO)
        ld   BC, format({%-11s},_cursor); 3:10      __INFO
        ld    A,(BC)        ; 1:7       __INFO
        xor  0x09           ; 2:7       __INFO
        ld  (BC),A          ; 1:7       __INFO})dnl


So the part that counts living neighbors looks like this:

    SCOLON(sum_neighbors,( addr -- sum ))
        ; [-1]
        DUP  LEFT
        ; [+1]
        SWAP RIGHT
        ; [+33]
        DUP PUSH_ADD(_w)
        DUP_PUSH_UGE_IF(_stop)
            PUSH_ADD(-_wh)
        THEN
        ; [+32]
        DUP  LEFT
        ; [+31]
        DUP  LEFT
        ; [-33]
        DUP PUSH_ADD(-2*_w)
        DUP_PUSH_ULT_IF(_screen)
            PUSH_ADD(_wh)
        THEN
        ; [-32]
        DUP  RIGHT
        ; [-31]
        DUP  RIGHT

        FETCH 
        SWAP FETCH ADD
        SWAP FETCH ADD
        SWAP FETCH ADD
        SWAP FETCH ADD
        SWAP FETCH ADD
        SWAP FETCH ADD
        SWAP FETCH ADD
        ; 16 bit --> 8 bit
        PUSH_AND(0xFF)
    SSEMICOLON

    
### Khammurapi

Khammurapi is a text-based strategy video game of land and resource management.

This is my modified version from sources:

- https://github.com/philspil66/Hamurabi (basic 1978)
- https://github.com/wzhishen/hammurabi (python 2012)

- https://codeberg.org/DW0RKiN/M4_FORTH/src/branch/master/Game/khammurapi.fth

![Khammurapi Game screenshot](https://codeberg.org/DW0RKiN/M4_FORTH/raw/branch/master/Game/khammurapi2.png)

- https://codeberg.org/DW0RKiN/M4_FORTH/src/branch/master/Game/khammurapi.m4

![Khammurapi Game screenshot](https://codeberg.org/DW0RKiN/M4_FORTH/raw/branch/master/Game/khammurapi.png)


Khammurapi is modified from 10 years to 12 years to increase your chances of success and still not overflow 16 bit numbers.
Each year you determine whether to buy or sell an acre of your land, how many bushels of wheat to set aside to feed your population, and how many bushels of grain to use for sowing for the next harvest. 
In exactly that order. Then random events occur that make your progress difficult.

#### Rules:

The game lasts 12 years (and not 10 like in the original to increase your chances of success), with a year being one round.

Each year, enter how many bushels of grain to allocate for buying (or selling if negative) acres of land.
The price of land varies from 17 to 26 bushels per acre for the year.

Each year, enter how many remaining bushels you set aside to feed the residents. 
Each citizen needs 20 bushels of grain per year. If he doesn't have it, he will starve to death. 
This version reduces the difficulty by having only half of the starving die. If more than 45% of the population dies in a famine that year, you will be overthrown and will not complete your 12 year term.

Each year, enter how many bushels of grain you have left over to plant next year's crops.
Each acre of land requires two bushels of grain (not one as in the original) to plant seeds.
One person can cultivate a maximum of 10 acres of land.
You cannot farm more acres than you own.

If you make it to Year 13, your results will be assessed.

#### Events you are up against:

There is a 15% chance that exactly half the population will die of plague in a given year.

Half of the remaining population that does not have enough food will die.

There is a 40% chance that your granaries (not your harvest yield) will be attacked by rats. 
Unlike the original version where you lose half or a quarter of your inventory, you'll likely lose less inventory here. 
It is calculated from the formula: BUSHELS OF GRAIN IN GRANARY / 2 to 31. So in the worst cases you will lose 50%,33%,25%,20%...etc. 
The median is 6% and the mean is 10%.

Harvest yield is 1 to 5 bushels per acre in the original. 
So if you want to break even, you have to have a yield of 3 bushels per acre. 
So on average you gain nothing by seeding because 2 are eaten by the person and 1 is lost by seeding. 
In this version, you need 2 bushels to seed one acre, so you must have a yield of at least 4 bushels. 
The yield probability per acre is adjusted from 1 to 5 to 0 to 8 and is calculated as the sum of two random values from 0 to 4. So on average you still get nothing from sowing, but the middle values are more likely.
At zero yield, expect a locust event.

#### Basic strategy:

Besides practicing multiplying by 10, 20 and dividing by two, you'll probably need an excel program for best strategy and good luck, or a crystal ball for fortune telling (load/save in ZX emulator). 
Because the game will constantly trip you up and you will get less bushels than you sow in the ground. 
You can't do without selling the land if you don't even have enough bushels of grain to feed the population and plant for the next year. 
Without selling acres at a high price like 24 to 26, you can't do without speculation of a price reduction next year. 
With the belief that rats will avoid your granaries this year.
